import PropTypes from 'prop-types';
import CustomPropTypes from "../../../storybook/utils/prop-types.js";
import useResponsiveValues from "../../../hooks/breakpoint/useResponsiveValues/index.js";
import cx from "../../../utils/css/cx.js";
import customizable from "../../../utils/css/customizable.js";
import getMarginClasses from "../../../utils/css/getMarginClasses.js";
import getColor from "../../../utils/colors/getColor.js";
import Block from './Block';
import legalStuffDescription from './content/legal-stuff';
import classes from './styles.module.scss';

/**
 * A standard footer to include at the bottom of a custom POLITICO interactive
 * story. This is generally used to include some legal disclaimers, a copyright
 * notice, and whatever additional credits or disclaimers are necessary for the
 * project. The "Legal Stuff" block is included by default, as is the copyright
 * notice, though you can disable the legal block and customize the copyright
 * notice with props.
 *
 * ```js
 *
 * import { InteractiveFooter } from '@politico/interactive-style';
 * ```
 */
const InteractiveFooter = props => {
  const {
    content,
    copyrightYear,
    copyright: customCopyright,
    disableLegalBlock,
    margin,
    bg,
    fg
  } = props;
  const marginValue = useResponsiveValues(margin);
  const margins = getMarginClasses(marginValue, {
    defaultSide: 't',
    sides: ['t']
  });
  const noContent = content.length === 0;
  const copyright = customCopyright || `© ${copyrightYear} POLITICO LLC`;
  return <footer className={cx(customizable('InteractiveFooter'), classes.footer, margins)} style={{
    '--InteractiveFooter-bg': getColor(bg),
    '--InteractiveFooter-fg': getColor(fg)
  }}>
      <div className={cx(customizable('InteractiveFooter__content'), classes.content)}>

        {content.map(a => <Block key={a.title} title={a.title} description={a.description} fullWidth={a.fullWidth} content={a.children} />)}

        {!disableLegalBlock && <Block title="Legal stuff" description={legalStuffDescription} fullWidth={noContent} />}

        <div className={cx(customizable('InteractiveFooter__copyright'), classes.copyright)}>
          {copyright}
        </div>

      </div>
    </footer>;
};
InteractiveFooter.defaultProps = {
  content: [],
  copyrightYear: 2023,
  copyright: undefined,
  disableLegalBlock: false,
  margin: '8',
  bg: undefined,
  fg: undefined
};
InteractiveFooter.propTypes = {
  /**
   * The blocks of content to include as sections in the footer. The standard
   * block contains a title and a Markdown-formatted description. The title
   * of each block is used as its key, so all titles must be unique. An
   * individual block can be flagged as `fullWidth` if it should span both
   * footer columns on desktop. Finally, a block can specify `children`, which
   * is used as the block's content instead of the basic title/description.
   * Specify `children` if you want to take full custom control of the block's
   * content. Note that even if you specify `children`, the block must still
   * have a unique `title` for use as a key.
   */
  content: PropTypes.arrayOf(PropTypes.shape({
    title: PropTypes.string.isRequired,
    description: PropTypes.string,
    fullWidth: PropTypes.bool,
    children: PropTypes.node
  })),
  /**
   * The year to use in the copyright at the bottom of the footer. This should
   * match the last year the project was published or updated.
   */
  copyrightYear: PropTypes.node,
  /**
   * The `copyright` replaces the default copyright notice entirely with
   * a custom block of content.
   */
  copyright: PropTypes.node,
  /**
   * Prevent the standard legal block from being included in the footer. Set
   * this to `true` if you want to include the legal stuff in your own content
   * directly.
   */
  disableLegalBlock: PropTypes.bool,
  /**
   * The top margin of the footer. The margin can be specified as one spacing
   * value or as an explicit `t` value. The value corresponds to levels of our
   * spacing scale (e.g., `"4"` corresponds to `$spacing-4`). Note that the
   * footer only supports a custom top margin. You can also provide
   * a responsive value; see [our guide on our standard styling props][utility]
   * for more details.
   *
   * [utility]: ./?path=/docs/guides-how-we-handle-styles--page#standard-styling-props
   */
  margin: CustomPropTypes.responsive(PropTypes.oneOfType([PropTypes.string, PropTypes.shape({
    t: PropTypes.string
  })])),
  /**
   * Sets the background color of the footer, which is `$altblack-90` by
   * default. This prop accepts either a color name provided by this library or
   * a CSS color value.
   */
  bg: PropTypes.string,
  /**
   * Sets the foreground color of the footer, which is `white` by default. This
   * prop accepts either a color name provided by this library or a CSS color
   * value.
   */
  fg: PropTypes.string
};
export default InteractiveFooter;